// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file Boundary.h */

#ifndef AMDIS_BOUNDARY_H
#define AMDIS_BOUNDARY_H

#include "Global.h"
#include "MemoryManager.h"
#include <map>

namespace AMDiS {

  template<typename T> class WorldVector;

  /// Contains boundary constants
  typedef enum {
    INTERIOR = 0,   /**< interior => no boundary (b = 0) */
    DIRICHLET = 1,  /**< dirichlet boundary (b > 0) */
    NEUMANN = -1,   /**< neumann boundary  (-100 < b < 0) */
    ROBIN = -100    /**< robin boundary (b <= -100) */
  } BoundaryConstants;

  /// Type specifier for the different boundary types 
  typedef signed int BoundaryType;

  // /** \ingroup Triangulation 
  //  * \brief
  //  * Holds information about the type of boundary associated to an edge/face,
  //  * and how new vertices are projected to the boundary in the case of curved
  //  * boundaries.
  // 
  // class Boundary
  // {
  // public:
  //   MEMORY_MANAGED(Boundary);

  //   /** \brief
  //    * constructor
  //   
  //   Boundary(BoundaryType type=0) { 
  //     bound = type; 
  //   };

  //   /** \brief
  //    * copy constructor
  //   
  //   Boundary(const Boundary& old) { bound = old.getBound(); };

  //   /** \brief
  //    * destructor
  //   
  //   virtual ~Boundary() {};

  //   /** \brief
  //    * assignment operator
  //   
  //   Boundary& operator=(const Boundary& old) { 
  //     if (this!=&old) bound = old.getBound(); 
  //     return *this; 
  //   };

  //   /** \brief
  //    * Returns
  //    * -true: if a new vertex should be projected to a curved boundary
  //    * -false: otherwise 
  //   
  //   virtual bool interpolateBoundary() { return false; };

  //   /** \brief
  //    * Projection to the curved boundary
  //    
  //   virtual void interpolateBoundary(WorldVector<double>& ) {};

  //   /** \brief
  //    * Returns \ref bound
  //   
  //   inline const BoundaryType getBound() const { return bound; };

  //   /** \brief
  //    * Returns
  //    * -true: is \ref bound is INTERIOR
  //    * -false: otherwise
  //   
  //   inline const bool isInterior() const {return  (bound == INTERIOR);};

  //   /** \brief
  //    * Returns
  //    * -true: is \ref bound is DIRICHLET
  //    * -false: otherwise
  //   
  //   inline const bool isDirichlet() const {return  (bound == DIRICHLET);};

  //   /** \brief
  //    * Returns
  //    * -true: is \ref bound is NEUMANN
  //    * -false: otherwise
  //   
  //   inline const bool isNeumann() const {return  (bound == NEUMANN);};

  //   /** \brief
  //    * Returns
  //    * -true: is \ref bound is ROBIN
  //    * -false: otherwise
  //   
  //   inline const bool isRobin() const {return  (bound == ROBIN);};

  //   /** \brief
  //    * Returns the new value of \ref bound with respect to its old value and
  //    * the value of bound_.
  //   
  //   BoundaryType newVal(const BoundaryType bound_);

  //   /** \brief
  //    * Returns the Boundary with given type from \ref boundaryMap.
  //   
  //   static Boundary* getBoundary(BoundaryType type);

  //   /** \brief
  //    * Adds Boundary b to \ref boundaryMap.
  //    
  //   //static void addBoundary(Boundary *b);

  // protected:
  //   /** \brief
  //    * type of this boundary
  //   
  //   BoundaryType bound;

  // protected:
  //   /** \brief
  //    * stl map of all existing boundaries.
  //   
  //   static ::std::map<BoundaryType, Boundary*> boundaryMap;
  // };

  BoundaryType newBound(BoundaryType oldBound, BoundaryType newBound);

}

#endif  // _Boundary_H_ 
