// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file ElInfo.h */

#ifndef AMDIS_ELINFOSTACK_H
#define AMDIS_ELINFOSTACK_H

// ============================================================================
// ===== includes =============================================================
// ============================================================================
#include <vector>
#include "ElInfo.h"

namespace AMDiS {

  // ============================================================================
  // ===== forward declarations =================================================
  // ============================================================================

  class Mesh;


  // ============================================================================
  // ===== class ElInfoStack ====================================================
  // ============================================================================

  /** \ingroup Traverse
   * \brief 
   * Stores a stack of ElInfo object. Is used by meshes for recursive mesh
   * traverse. The use of a stack is cheaper than allocating the ElInfo objects
   * at every recursive step.
   */

  class ElInfoStack
  {
  public:
    /** \brief
     * Constructer, creates the stack.
     */
    ElInfoStack(Mesh *mesh);
    
    /** \brief
     * Destructor, deletes all ElInfos on the stack.
     */
    ~ElInfoStack();

    /** \brief
     * Get a new element from the stack an increase the stack position.
     */
    ElInfo* getNextElement();

    /** \brief
     * Decrease the stack position.
     */
    void getBackElement();

    /** \brief
     * Returns a pointer to the currently used element of the stack.
     */
    ElInfo* getCurrentElement();

  protected:
    /** \brief
     * The mesh on which the traverse is done.
     */
    Mesh *mesh_;

    /** \brief
     * The stack of pointers to ElInfo objects.
     */
    std::vector<ElInfo*> elInfoStack_;

    /** \brief
     * Current position (depth) of the recursive mesh traverse.
     */
    int stackPosition_;
  };
  
}


#endif  // AMDIS_ELINFOSTACK_H
